@@ -22,8 +22,12 @@
  */
 
 #include <config.h>
+
+#include <gdkconfig.h>
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
+#endif
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <libgnome/gnome-macros.h>
@@ -33,6 +37,8 @@
 #include "eog-scroll-view.h"
 #include "cursors.h"
 #include "eog-config-keys.h"
+#include "eog-job-manager.h"
+#include "eog-image-cache.h"
 
 GNOME_CLASS_BOILERPLATE (EogFullScreen,
 			 eog_full_screen,
@@ -40,8 +46,7 @@ GNOME_CLASS_BOILERPLATE (EogFullScreen,
 			 GTK_TYPE_WINDOW);
 
 #define POINTER_HIDE_DELAY  2   /* hide the pointer after 2 seconds */
-static GQuark FINISHED_QUARK = 0; 
-static GQuark FAILED_QUARK = 0; 
+static GQuark JOB_ID_QUARK = 0; 
 
 typedef enum {
 	EOG_DIRECTION_FORWARD,
@@ -63,6 +68,9 @@ struct _EogFullScreenPrivate
 	
 	/* current visible image iterator */
 	EogIter *current;
+	/* iter of currently preloaded image */
+	EogIter *preload;
+	EogImageCache *cache;
 
 	/* if the mouse pointer is hidden */
 	gboolean cursor_hidden;
@@ -80,12 +88,17 @@ struct _EogFullScreenPrivate
 	gint   switch_timeout;      /* in seconds */
 	guint  switch_timeout_id;
 	gboolean switch_pause;      
+	gboolean slide_show;      
+
+	/* timeout for screensaver blocking */
+	gint   activity_timeout_id;
+
+	/* skip loading on fast subsequent key presses */
+	guint32  last_key_press_time;
+	guint    display_id;
 };
 
-static void connect_image_callbacks (EogFullScreen *fs, EogImage *image);
-static void disconnect_image_callbacks (EogImage *image);
-static void prepare_load_image (EogFullScreen *fs, EogIter *iter);
-static void show_next_image (EogFullScreen *fs);
+static guint prepare_load_image (EogFullScreen *fs, EogIter *iter);
 
 static gboolean
 check_cursor_hide (gpointer data)
@@ -145,11 +158,138 @@ eog_full_screen_motion (GtkWidget *widget, GdkEventMotion *event)
 	return result;
 }
 
+static void
+cancel_load_by_iter (EogImageList *list, EogIter *iter)
+{
+	EogImage *image;
+	guint job_id = 0;
+	
+	image = eog_image_list_get_img_by_iter (list, iter);
+	if (image != NULL) {
+		job_id = GPOINTER_TO_INT (g_object_get_qdata (G_OBJECT (image), JOB_ID_QUARK));
+		g_object_unref (image);
+	}
+	
+	if (job_id > 0) {
+		eog_job_manager_cancel_job (job_id);
+	}
+}
+
+static EogIter*
+get_next_iter_in_direction (EogImageList *list, EogIter *iter, EogDirection direction)
+{
+	EogIter *next = NULL;
+	gboolean success = FALSE;
+
+	next = eog_image_list_iter_copy (list, iter);
+
+	if (direction == EOG_DIRECTION_FORWARD) {
+		success = eog_image_list_iter_next (list, next, TRUE);
+	}
+	else {
+		success = eog_image_list_iter_prev (list, next, TRUE);
+	}
+
+	if (!success) {
+		g_free (next);
+		next = NULL;
+	}
+
+	return next;
+}
+
+static void
+display_image_on_screen (EogFullScreen *fs, EogIter *iter) 
+{
+	EogFullScreenPrivate *priv;
+	EogImage *image;
+	EogIter *next_preload;
+
+	priv = fs->priv;
+
+	if (priv->current != iter) {
+		if (priv->current != NULL) {
+			g_free (priv->current);
+		}
+		priv->current = eog_image_list_iter_copy (priv->list, iter);
+	}
+
+	/* get image */
+	image = eog_image_list_get_img_by_iter (priv->list, priv->current);
+
+	if (eog_image_has_data (image, EOG_IMAGE_DATA_IMAGE)) {
+		eog_image_cache_add (priv->cache, image);
+
+		/* image is already loaded */
+		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->view), image);
+		
+		/* preload next image in current direction */
+		next_preload = get_next_iter_in_direction (priv->list, priv->current, priv->direction);
+
+		if (next_preload != NULL && 
+		    !eog_image_list_iter_equal (priv->list, priv->preload, next_preload))
+		{
+			cancel_load_by_iter (priv->list, priv->preload);
+			
+			if (priv->preload != NULL)
+				g_free (priv->preload);
+			
+			priv->preload = next_preload;
+			prepare_load_image (fs, priv->preload);
+		}
+		else if (next_preload != NULL) {
+			g_free (next_preload);	
+		}
+	}
+	else if (g_object_get_qdata (G_OBJECT (image), JOB_ID_QUARK) == NULL) { 
+		/* if the image is not loading already */
+		prepare_load_image (fs, priv->current);
+	}
+
+	/* FIXME: pause auto advance */
+
+	priv->switch_time_counter = 0;
+	g_object_unref (image);
+}
+
+static gboolean
+delayed_display_image_on_screen (gpointer data) 
+{
+	EogFullScreen *fs;
+	EogFullScreenPrivate *priv;
+
+	fs = EOG_FULL_SCREEN (data);
+	priv = fs->priv;
+
+	display_image_on_screen (fs, priv->current);
+
+	priv->display_id = 0;
+	return FALSE;
+}
+
+static gboolean
+is_loop_end (EogImageList *list, EogIter *current, EogDirection direction)
+{
+	gboolean is_end = FALSE;
+
+	int pos = eog_image_list_get_pos_by_iter (list, current);
+
+	if (direction == EOG_DIRECTION_FORWARD) {
+		is_end = (pos == (eog_image_list_length (list) - 1));
+	}
+	else {
+		is_end = (pos == 0);
+	}
+	
+	return is_end;
+}
+
 static gboolean
 check_automatic_switch (gpointer data)
 {
 	EogFullScreen *fs;
 	EogFullScreenPrivate *priv;
+	EogIter *next = NULL;
 
 	fs = EOG_FULL_SCREEN (data);
 	priv = fs->priv;
@@ -161,12 +301,58 @@ check_automatic_switch (gpointer data)
 
 	if (priv->switch_time_counter == priv->switch_timeout) {
 		priv->direction = EOG_DIRECTION_FORWARD;
-		show_next_image (fs);
+
+		if (priv->loop || !is_loop_end (priv->list, priv->current, priv->direction)) {
+			next = get_next_iter_in_direction (priv->list, priv->current, priv->direction);
+		}
+
+		if (next != NULL){
+			display_image_on_screen (fs, next);
+			g_free (next);
+		}
+		else {
+			gtk_widget_hide (GTK_WIDGET (fs));
+		}
 	}
 
 	return TRUE;
 }
 
+/* Works only for xscreensaver */
+static gboolean
+disable_screen_saver (gpointer data)
+{
+	EogFullScreenPrivate *priv;
+	gboolean success = TRUE;
+	char *argv[] = { "xscreensaver-command", "-deactivate", NULL };
+
+	priv = EOG_FULL_SCREEN (data)->priv;
+	
+	if (!priv->switch_pause) {
+		GSpawnFlags flags = G_SPAWN_SEARCH_PATH | 
+		                    G_SPAWN_STDOUT_TO_DEV_NULL | 
+		                    G_SPAWN_STDERR_TO_DEV_NULL;
+		
+		success = g_spawn_sync (NULL,  /* working directory */
+								argv,  /* command */
+								NULL,  /* environment */
+								flags, /* flags */
+								NULL,  /* child setup func */
+								NULL,  /* function data */
+								NULL,  /* standard output */
+		                        NULL,  /* standard error */
+								NULL,  /* exit status */
+								NULL); /* GError */
+	}
+	
+	if (!success) {
+		priv->activity_timeout_id = 0;
+		g_print ("Disable xscreensaver failed.\n");		
+	}
+	
+	return success;
+}
+
 /* Show handler for the full screen view */
 static void
 eog_full_screen_show (GtkWidget *widget)
@@ -187,15 +373,21 @@ eog_full_screen_show (GtkWidget *widget)
 
 	priv->cursor_hidden = FALSE;
 	priv->mouse_move_counter = 0;
-        priv->hide_timeout_id = g_timeout_add (1000 /* every second */,
+	priv->hide_timeout_id = g_timeout_add (1000 /* every second */,
 					       check_cursor_hide,
 					       fs);
 
 	priv->switch_timeout_id = 0;
-	if (priv->switch_timeout > 0) {
+	priv->activity_timeout_id = 0;
+	if (priv->slide_show) {
 		priv->switch_timeout_id = g_timeout_add (1000 /* every second */,
 							 check_automatic_switch,
 							 fs);
+
+		/* disable screen saver */
+		priv->activity_timeout_id = g_timeout_add (6000 /* every minute */,
+							   disable_screen_saver,
+							   fs);
 	}
 }
 
@@ -212,120 +404,19 @@ eog_full_screen_hide (GtkWidget *widget)
 		gdk_keyboard_ungrab (GDK_CURRENT_TIME);
 	}
 
-	if (fs->priv->switch_timeout_id > 0) {
+	if (fs->priv->slide_show) {
 		g_source_remove (fs->priv->switch_timeout_id);
+		fs->priv->switch_timeout_id = 0;
+	}
+	
+	if (fs->priv->activity_timeout_id > 0) {
+		g_source_remove (fs->priv->activity_timeout_id);
+		fs->priv->activity_timeout_id = 0;
 	}
 
 	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, hide, (widget));
 }
 
-static void 
-show_nth_image (EogFullScreen *fs, unsigned int nth)
-{
-	EogFullScreenPrivate *priv;
-	EogImage *image;
-	EogIter *iter_copy;
-	EogIter *nth_iter;
-	gboolean success = FALSE;
-	
-	priv = fs->priv;
-
-	/* point the current iterator to the nth image of the list */
-	nth_iter = eog_image_list_get_iter_by_pos (priv->list, nth);
-
-	if (nth_iter != NULL) {
-		if (priv->current != NULL) {
-			g_free (priv->current);
-		}
-		priv->current = nth_iter;
-
-		/* view nth image */
-		image = eog_image_list_get_img_by_iter (priv->list, priv->current);
-		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->view), image);
-		priv->switch_time_counter = 0;
-		g_object_unref (image);
-	}
-	else {
-		/* quit diashow */
-		gtk_widget_hide (GTK_WIDGET (fs));
-		return;
-	}
-
-	/* preload next image in current direction */
-	iter_copy = eog_image_list_iter_copy (priv->list, priv->current);
-	if (priv->direction == EOG_DIRECTION_FORWARD) {
-		success = eog_image_list_iter_next (priv->list, iter_copy, TRUE);
-	}
-	else {
-		success = eog_image_list_iter_prev (priv->list, iter_copy, TRUE);
-	}
-
-	if (success) {
-		prepare_load_image (fs, iter_copy);
-	}
-
-	g_free (iter_copy);	
-}
-
-static void
-show_next_image (EogFullScreen *fs)
-{
-	EogFullScreenPrivate *priv;
-	EogImage *image;
-	EogIter *iter_copy;
-	gboolean success = FALSE;
-	
-	priv = fs->priv;
-
-	/* point the current iterator to the next image in the choosen direction */
-	if (priv->direction == EOG_DIRECTION_FORWARD) {
-		success = eog_image_list_iter_next (priv->list, priv->current, TRUE);
-
-		/* quit the diashow, if the user don't want to loop
-		 * and we are pointing to the first image again */
-		if (!priv->loop && eog_image_list_iter_equal (priv->list, priv->first_iter, priv->current)) {
-			success = FALSE;
-		}
-	}
-	else {
-		/* quit the diashow, if the user don't want to loop
-		 * and we are pointing currently to the first image */
-		if (!priv->loop && eog_image_list_iter_equal (priv->list, priv->first_iter, priv->current)) {
-			success = FALSE;
-		}
-		else {
-			success = eog_image_list_iter_prev (priv->list, priv->current, TRUE);
-		}
-	}
-
-	if (success) {
-		/* view next image */
-		image = eog_image_list_get_img_by_iter (priv->list, priv->current);
-		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->view), image);
-		priv->switch_time_counter = 0;
-		g_object_unref (image);
-	}
-	else {
-		/* quit diashow */
-		gtk_widget_hide (GTK_WIDGET (fs));
-		return;
-	}
-
-	/* preload next image in current direction */
-	iter_copy = eog_image_list_iter_copy (priv->list, priv->current);
-	if (priv->direction == EOG_DIRECTION_FORWARD) {
-		success = eog_image_list_iter_next (priv->list, iter_copy, TRUE);
-	}
-	else {
-		success = eog_image_list_iter_prev (priv->list, iter_copy, TRUE);
-	}
-
-	if (success) {
-		prepare_load_image (fs, iter_copy);
-	}
-
-	g_free (iter_copy);
-}
 
 /* Key press handler for the full screen view */
 static gint
@@ -335,6 +426,7 @@ eog_full_screen_key_press (GtkWidget *widget, GdkEventKey *event)
 	EogFullScreen *fs;
 	gint handled;
 	gboolean do_hide;
+	EogIter *next = NULL;
 
 	fs = EOG_FULL_SCREEN (widget);
 	priv = fs->priv;
@@ -343,10 +435,21 @@ eog_full_screen_key_press (GtkWidget *widget, GdkEventKey *event)
 	do_hide = FALSE;
 
 	switch (event->keyval) {
+	case GDK_F11:
+		if (!priv->slide_show) {
+			do_hide = handled = TRUE;
+		}
+		break;
+
+	case GDK_F5:
+		if (priv->slide_show) {
+			do_hide = handled = TRUE;
+		}
+		break;
+		
 	case GDK_Q:
 	case GDK_q:
 	case GDK_Escape:
-	case GDK_F11:
 	case SunXK_F36:
 		do_hide = handled = TRUE;
 		break;
@@ -362,10 +465,11 @@ eog_full_screen_key_press (GtkWidget *widget, GdkEventKey *event)
 	case GDK_Right:
 	case GDK_Down:
 	case GDK_Page_Down:
-		if (eog_image_list_length (priv->list) > 1) {
+		if (!priv->slide_show && eog_image_list_length (priv->list) > 1) {
 			priv->direction = EOG_DIRECTION_FORWARD;
-
-			show_next_image (fs);
+			
+			next = get_next_iter_in_direction (priv->list, priv->current, 
+							   priv->direction);
 			handled = TRUE;
 		}
 		break;
@@ -379,28 +483,52 @@ eog_full_screen_key_press (GtkWidget *widget, GdkEventKey *event)
 	case GDK_Left:
 	case GDK_Up:
 	case GDK_Page_Up:
-		if (eog_image_list_length (priv->list) > 1) {
+		if (!priv->slide_show && eog_image_list_length (priv->list) > 1) {
 			priv->direction = EOG_DIRECTION_BACKWARD;
 
-			show_next_image (fs);
+			next = get_next_iter_in_direction (priv->list, priv->current, priv->direction);
 			handled = TRUE;
 		}
 		break;
 
 	case GDK_Home:
-		if (eog_image_list_length (priv->list) > 1) {
-			show_nth_image (fs, 0);
+		if (!priv->slide_show && eog_image_list_length (priv->list) > 1) {
+			next = eog_image_list_get_iter_by_pos (priv->list, 0);
 			handled = TRUE;
 		}
 		break;
 
 	case GDK_End:
-		if (eog_image_list_length (priv->list) > 1) {
-			show_nth_image (fs, eog_image_list_length (priv->list) - 1);
+		if (!priv->slide_show && eog_image_list_length (priv->list) > 1) {
+			next = eog_image_list_get_iter_by_pos (priv->list, eog_image_list_length (priv->list) - 1);
 			handled = TRUE;
 		}
 	};
 	
+	if (next != NULL) {  
+		if (!priv->loop && is_loop_end (priv->list, priv->current, priv->direction)) {
+			do_hide = TRUE;
+		}
+		else if ((event->time - priv->last_key_press_time) < 1000) {
+			/* just update iterator and delay loading of current iter */
+			if (priv->current != NULL) {
+				g_free (priv->current);
+			}
+			priv->current = eog_image_list_iter_copy (priv->list, next);
+
+			if (priv->display_id != 0) {
+				g_source_remove (priv->display_id);
+			}
+			priv->display_id = g_timeout_add (500, delayed_display_image_on_screen, fs);
+		}
+		else {
+			display_image_on_screen (fs, next);
+		}
+
+		priv->last_key_press_time = event->time;
+		g_free (next);
+	}
+
 	if (do_hide) {
 		gtk_widget_hide (widget);
 	}
@@ -417,8 +545,6 @@ eog_full_screen_destroy (GtkObject *object)
 {
 	EogFullScreen *fs;
 	EogFullScreenPrivate *priv;
-	EogIter *iter;
-	gboolean success;
 	
 	g_return_if_fail (EOG_IS_FULL_SCREEN (object));
 
@@ -436,17 +562,6 @@ eog_full_screen_destroy (GtkObject *object)
 	}
 
 	if (priv->list != NULL) {
-		/* remove all callback handlers */
-		iter = eog_image_list_get_first_iter (priv->list);
-		success = (iter != NULL);
-		while (success) {
-			EogImage *image;
-			image = eog_image_list_get_img_by_iter (priv->list, iter);
-			disconnect_image_callbacks (image);
-			g_object_unref (image);
-			success = eog_image_list_iter_next (priv->list, iter, FALSE);
-		}
-		
 		/* free list */
 		g_object_unref (priv->list);
 		priv->list = NULL;
@@ -457,6 +572,11 @@ eog_full_screen_destroy (GtkObject *object)
 		priv->first_iter = NULL;
 	}
 
+	if (priv->cache != NULL) {
+		g_object_unref (priv->cache);
+		priv->cache = NULL;
+	}
+
 	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }	
 
@@ -504,22 +624,9 @@ static void
 eog_full_screen_instance_init (EogFullScreen *fs)
 {
 	EogFullScreenPrivate *priv;
-	GdkScreen *screen;
-	GdkRectangle geometry;
-	int monitor;
-
 	priv = g_new0 (EogFullScreenPrivate, 1);
 	fs->priv = priv;
 
-	screen = gtk_window_get_screen (GTK_WINDOW (fs));
-	monitor = gdk_screen_get_monitor_at_window (screen, GTK_WIDGET (fs)->window);
-	gdk_screen_get_monitor_geometry (screen, monitor, &geometry);
-
-	gtk_window_set_default_size (GTK_WINDOW (fs),
-				     geometry.width,
-				     geometry.height);
-	gtk_window_move (GTK_WINDOW (fs), geometry.x, geometry.y);
-
 	priv->loop = TRUE;
 	priv->first_iter = NULL;
 
@@ -527,104 +634,130 @@ eog_full_screen_instance_init (EogFullScreen *fs)
 	priv->switch_timeout_id = 0;
 	priv->switch_time_counter = 0;
 	priv->switch_pause = FALSE;
+
+	priv->preload = NULL;
+	priv->cache = eog_image_cache_new (3); /* maximum number of cached images */
+
+	priv->last_key_press_time = 0;
+	priv->display_id = 0;
 }
 
 static void
-image_loading_finished_cb (EogImage *image, gpointer data)
+eog_full_screen_set_geometry (EogFullScreen *fs, GtkWindow *parent)
 {
-	EogFullScreen *fs;
 	EogFullScreenPrivate *priv;
-	EogIter *iter_copy;
-
-	g_return_if_fail (EOG_IS_FULL_SCREEN (data));
+	GdkScreen *screen;
+	GdkRectangle geometry;
+	int monitor;
 	
-	fs = EOG_FULL_SCREEN (data);
-
 	priv = fs->priv;
 
-	disconnect_image_callbacks (image);
+	screen = gtk_window_get_screen (GTK_WINDOW (fs));
+	monitor = gdk_screen_get_monitor_at_window (screen,
+			GTK_WIDGET (parent)->window);
+	gdk_screen_get_monitor_geometry (screen, monitor, &geometry);
 
-	if (priv->first_image) {
-		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->view), image);
-		priv->first_image = FALSE;
+	gtk_window_set_default_size (GTK_WINDOW (fs),
+				     geometry.width,
+				     geometry.height);
+	gtk_window_move (GTK_WINDOW (fs), geometry.x, geometry.y);
+}
 
-		iter_copy = eog_image_list_iter_copy (priv->list, priv->current);
-		if (eog_image_list_iter_next (priv->list, iter_copy, TRUE)) {
-			prepare_load_image (fs, iter_copy);
-		}
-		g_free (iter_copy);
+typedef struct {
+	EogFullScreen *fs;
+	EogImage      *image;
+} JobImageLoadData;
 
-		iter_copy = eog_image_list_iter_copy (priv->list, priv->current);
-		if (eog_image_list_iter_prev (priv->list, iter_copy, TRUE)) {
-			prepare_load_image (fs, iter_copy);
-		}
-		g_free (iter_copy);
+static void
+job_image_load_do (EogJob *job, gpointer data, GError **error)
+{
+	EogImage *image = ((JobImageLoadData*) data)->image;
+
+	eog_image_load (image, EOG_IMAGE_DATA_IMAGE, job, error);
+}
+
+static void
+job_image_load_finished  (EogJob *job, gpointer data, GError *error)
+{
+	EogFullScreen *fs = ((JobImageLoadData*) data)->fs;
+	EogFullScreenPrivate *priv = fs->priv;
+	EogImage *image = ((JobImageLoadData*) data)->image;
+	EogImage *current_img;
+	EogImage *preload_img;
+
+	if (eog_job_get_success (job))
+		eog_image_cache_add (priv->cache, image);
+
+	/* check if the finished image is the one we want to display now */
+	current_img = eog_image_list_get_img_by_iter (priv->list, priv->current);
+	if (current_img == image) {
+		if (eog_job_get_success (job))
+			eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->view), image);
+		else 
+			eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->view), NULL);
+	}
+	if (current_img != NULL) 
+		g_object_unref (current_img);
+	
+	/* check if the finished image is the one we want to preload */
+	preload_img = eog_image_list_get_img_by_iter (priv->list, priv->preload);
+	if (preload_img == image) {
+		g_free (priv->preload);
+		priv->preload = NULL;
+	}
+	if (preload_img != NULL)
+		g_object_unref (preload_img);
 		
-	}
+	/* clean up, see prepare_load_image */
+	g_object_set_qdata (G_OBJECT (image), JOB_ID_QUARK, NULL);
+	eog_image_data_unref (image);
 }
 
 static void
-image_loading_failed_cb (EogImage *image, const char *message, gpointer data)
+job_image_load_cancel (EogJob *job, gpointer data)
 {
-	disconnect_image_callbacks (image);
+	EogImage *image = ((JobImageLoadData*) data)->image;
 
-	/* FIXME: What should we do if an image loading failed? */
+	eog_image_cancel_load (image);
 }
 
-static void
-disconnect_image_callbacks (EogImage *image)
-{
-	int id;
-
-	g_return_if_fail (EOG_IS_IMAGE (image));
-
-	if (g_object_get_qdata (G_OBJECT (image), FINISHED_QUARK) != NULL) { 
-		id = GPOINTER_TO_INT (g_object_get_qdata (G_OBJECT (image), FINISHED_QUARK));
-		g_signal_handler_disconnect (G_OBJECT (image), id);
-		g_object_set_qdata (G_OBJECT (image), FINISHED_QUARK, NULL);
-	}
-
-	if (g_object_get_qdata (G_OBJECT (image), FAILED_QUARK) != NULL) {
-		id = GPOINTER_TO_INT (g_object_get_qdata (G_OBJECT (image), FAILED_QUARK));
-		g_signal_handler_disconnect (G_OBJECT (image), id);
-		g_object_set_qdata (G_OBJECT (image), FAILED_QUARK, NULL);
-	}
-}
-
-static void 
-connect_image_callbacks (EogFullScreen *fs, EogImage *image)
-{
-	int id;
-
-	g_return_if_fail (EOG_IS_IMAGE (image));
-
-	if (g_object_get_qdata (G_OBJECT (image), FINISHED_QUARK) == NULL) {
-		id = g_signal_connect (G_OBJECT (image), "loading_finished", 
-				       G_CALLBACK (image_loading_finished_cb), fs);
-		g_object_set_qdata (G_OBJECT (image), FINISHED_QUARK, GINT_TO_POINTER (id));
-	}
-
-	if (g_object_get_qdata (G_OBJECT (image), FAILED_QUARK) == NULL) {
-		id = g_signal_connect (G_OBJECT (image), "loading_failed",
-				       G_CALLBACK (image_loading_failed_cb), fs);
-		g_object_set_qdata (G_OBJECT (image), FAILED_QUARK, GINT_TO_POINTER (id));
-	}
-}
-
-static void
+static guint
 prepare_load_image (EogFullScreen *fs, EogIter *iter)
 {
 	EogFullScreenPrivate *priv;
 	EogImage *image;
+	EogJob *job;
+	guint job_id = 0;
+	JobImageLoadData *ld;
 
 	priv = fs->priv;
 	
 	image = eog_image_list_get_img_by_iter (priv->list, iter);
-	connect_image_callbacks (fs, image);
+	eog_image_data_ref (image);
+	g_object_unref (image); /* eog_image_list_get_img_by_iter ref'ed the image too */
 
-	eog_image_load (image, EOG_IMAGE_LOAD_COMPLETE);
+	if (eog_image_has_data (image, EOG_IMAGE_DATA_IMAGE)) {
+		eog_image_data_unref (image);
+		return 0;
+	}
 
-	g_object_unref (image);
+	ld = g_new0 (JobImageLoadData, 1);
+	ld->image = image;
+	ld->fs = fs;
+	
+	job = eog_job_new_full (ld,
+				job_image_load_do,
+				job_image_load_finished,
+				job_image_load_cancel,
+				NULL,
+				g_free);
+	
+	job_id = eog_job_get_id (job);
+	g_object_set_qdata (G_OBJECT (image), JOB_ID_QUARK, GUINT_TO_POINTER (job_id));
+	
+	eog_job_manager_add (job);
+
+	return job_id;
 }
 
 static void
@@ -648,23 +781,18 @@ prepare_data (EogFullScreen *fs, EogImageList *image_list, EogImage *start_image
 
 	/* special case if we only have one image */
 	if (eog_image_list_length (image_list) == 1) {
-		EogImage *single_image;
-		single_image = eog_image_list_get_img_by_iter (image_list, priv->current);
-		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->view), 
-					   single_image);
-		g_object_unref (single_image);
-		priv->switch_timeout = 0; /* disable automatic switching always */
-	}
-	else {
-		priv->direction = EOG_DIRECTION_FORWARD;
-		priv->first_iter = eog_image_list_get_first_iter (image_list);
-		priv->first_image = TRUE;
-		prepare_load_image (fs, priv->current);
+		priv->slide_show = FALSE; /* disable automatic switching always */
 	}
+
+	priv->direction = EOG_DIRECTION_FORWARD;
+	priv->first_iter = eog_image_list_get_first_iter (image_list);
+
+	display_image_on_screen (fs, priv->current);
 }
 
 GtkWidget *
-eog_full_screen_new (EogImageList *image_list, EogImage *start_image)
+eog_full_screen_new (GtkWindow *parent,
+		EogImageList *image_list, EogImage *start_image, gboolean slide_show)
 {
 	EogFullScreen *fs;
 	EogFullScreenPrivate *priv;
@@ -677,8 +805,11 @@ eog_full_screen_new (EogImageList *image_list, EogImage *start_image)
 	g_return_val_if_fail (image_list != NULL, NULL);
 
 	fs = g_object_new (EOG_TYPE_FULL_SCREEN, 
-			   "type", GTK_WINDOW_POPUP, NULL);
+			   "type", GTK_WINDOW_POPUP,
+			   NULL);
 	priv = fs->priv;
+	/* bind to a particular monitor */
+	eog_full_screen_set_geometry (fs, parent);
 
 	widget = eog_scroll_view_new ();
 	priv->view = widget;
@@ -699,12 +830,11 @@ eog_full_screen_new (EogImageList *image_list, EogImage *start_image)
 	gtk_widget_show (widget);
 	gtk_container_add (GTK_CONTAINER (fs), widget);
 
-	if (FINISHED_QUARK == 0)
-		FINISHED_QUARK = g_quark_from_static_string ("EogFullScreen Finished Callback");
-	
-	if (FAILED_QUARK == 0) 
-		FAILED_QUARK = g_quark_from_static_string ("EogFullScreen Failed Callback");
+	if (JOB_ID_QUARK == 0)
+		JOB_ID_QUARK = g_quark_from_static_string ("EogFullScreen Load Job ID");
 
+	priv->slide_show = slide_show;
+	
 	/* read configuration */
 	client = gconf_client_get_default ();
 	priv->loop = gconf_client_get_bool (client, EOG_CONF_FULLSCREEN_LOOP, NULL);
@@ -725,7 +855,11 @@ eog_full_screen_new (EogImageList *image_list, EogImage *start_image)
 gboolean
 eog_full_screen_enable_SunF36 (void)
 {
+#ifdef GDK_WINDOWING_X11
 	return (XKeysymToKeycode (GDK_DISPLAY (), SunXK_F36) != 0);
+#else
+	return FALSE;
+#endif
 }
 
 EogImage*  
